@@ -1,5 +1,5 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003 Hewlett-Packard Co
+   Copyright (C) 2003-2004 Hewlett-Packard Co
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
@@ -31,6 +31,8 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #include <signal.h>
 #include <stdlib.h>
 
+#include "jmpbuf.h"
+
 #if UNW_TARGET_IA64
 # include "ia64/rse.h"
 #endif
@@ -38,8 +40,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 void
 _longjmp (jmp_buf env, int val)
 {
-  extern int _UI_siglongjmp_cont;
-  sigset_t current_mask;
+  extern int _UI_longjmp_cont;
   unw_context_t uc;
   unw_cursor_t c;
   unw_word_t sp;
@@ -52,7 +53,7 @@ _longjmp (jmp_buf env, int val)
     {
       if (unw_get_reg (&c, UNW_REG_SP, &sp) < 0)
 	abort ();
-      if (sp != wp[0])
+      if (sp != wp[JB_SP])
 	continue;
 
 #if UNW_TARGET_IA64
@@ -67,35 +68,23 @@ _longjmp (jmp_buf env, int val)
 	sol = (pfs >> 7) & 0x7f;
 	bsp = ia64_rse_skip_regs (bsp, sol);
 
-	if (bsp != wp[2])
+	if (bsp != wp[JB_BSP])
 	  continue;
       }
 #endif
 
-
       /* found the right frame: */
 
-      if (sigprocmask (SIG_BLOCK, NULL, &current_mask) < 0)
-	abort ();
-
-      assert (UNW_NUM_EH_REGS >= 4);
-
-      if (unw_set_reg (&c, UNW_REG_EH + 0, wp[1]) < 0
-	  || unw_set_reg (&c, UNW_REG_EH + 1, val) < 0
-	  || unw_set_reg (&c, UNW_REG_EH + 2,
-			  ((unw_word_t *) &current_mask)[0]) < 0
-	  || unw_set_reg (&c, UNW_REG_IP,
-			  (unw_word_t) &_UI_siglongjmp_cont))
-	abort ();
-
-      if (_NSIG > 8 * sizeof (unw_word_t))
+      if (UNW_NUM_EH_REGS >= 4)
 	{
-	  if (_NSIG > 16 * sizeof (unw_word_t))
-	    abort ();
-	  if (unw_set_reg (&c, UNW_REG_EH + 3,
-			   ((unw_word_t *) &current_mask)[1]) < 0)
+	  if (unw_set_reg (&c, UNW_REG_EH + 0, wp[JB_RP]) < 0
+	      || unw_set_reg (&c, UNW_REG_EH + 1, val) < 0
+	      || unw_set_reg (&c, UNW_REG_IP,
+			      (unw_word_t) &_UI_longjmp_cont))
 	    abort ();
 	}
+      else
+	abort ();
 
       unw_resume (&c);
 
